<?php

namespace BulkUpload\Handlers;

use BulkUpload\Country as CountryObject;
use BulkUpload\Error\Error;

class Country {

    private array $countries;
    private \collection $db;

    public function __construct()
    {
        $this->setCountries();
    }

    /**
     * Get the value of countries
     */ 
    public function getCountries()
    {
        return $this->countries;
    }

    /**
     * Sets the list of countries or retrives an error.
     *
     * @return Error|boolean
     */
    private function setCountries():Error|bool{

        $results = $this->db->search('ng_country', limit:1);

        if($results['cant_rows'] < 1){
            return new Error('CountryManager', 'There is no registry in the database with the list of countries');
        }

        $this->countries = json_decode($results[0]['body'], true);
        return true;

    }

    /**
     * Retrieves a list of cca2 => countriAlternativeNames list to use for validations.
     *
     * @return array
     */
    public function getValidationCountryList():array{

        $countries = [];

        foreach ($this->countries as $decoded) {
            //TODO: Instanciar los objetos de tipo country
            $countryTranslations = [];
            array_push($countryTranslations, $decoded['cca2']);
            array_push($countryTranslations, $decoded['name']);
            array_push($countryTranslations, $decoded['cca3']);
            array_push($countryTranslations, $decoded['cioc']);
            array_push($countryTranslations, $decoded['ccn3']);
            array_push($countryTranslations, strtolower($decoded['name']));
            array_push($countryTranslations, strtolower($decoded['cca2']));
            array_push($countryTranslations, strtolower($decoded['cca3']));
            array_push($countryTranslations, strtolower($decoded['cioc']));
            array_push($countryTranslations, strtolower($decoded['ccn3']));
            foreach ($decoded['translations'] as $value) {
                array_push($countryTranslations, $value['common']);
                array_push($countryTranslations, strtolower($value['common']));
            }
    
            $countries[$decoded['cca2']] = $countryTranslations;
        }

        return $countries;

    }

    static public function getPublicCountryList():array{

        $countryManager = new Country();
        return $countryManager->getValidationCountryList();

    }

}