<?php

namespace BulkUpload\Terms;

use BulkUpload\Error\Error;
use BulkUpload\Interfaces\TermCrudInterface;
use BulkUpload\Interfaces\TermInterface;

class Category implements TermInterface{

    public int $term_id;
    public int $term_taxonomy_id;
    public string $name;
    public string $slug;
    public string $taxonomy = "product_cat";
    public ?int $count;

    public int $parent = 0;


    /**
     * Instance of the database manager
     *
     * @var \collection
     */
    private \collection $db;

    /**
     * Instances a new Category class with a collection propertie to manage the database.
     */
    public function __construct(array $category_data)
    {
        $this->term_id = $category_data['term_id'];
        $this->term_taxonomy_id = $category_data['term_taxonomy_id'];
        $this->name = $category_data['name'];
        $this->slug = $category_data['slug'];
        $this->count = $category_data['count'];
        $this->parent = $category_data['parent'];
        $this->db = new \collection;
    }

    /**
     * Returns the term id.
     *
     * @return integer
     */
    public function getTermTermId()
    {
        return $this->term_id;
    }

    /**
     * Returns the term name.
     *
     * @return string
     */
    public function getTermName()
    {
        return $this->name;
    }

    /**
     * Returns a Category|Tag class that matches with the provided id.
     *
     * @param integer $term_id
     * @return Category|Error
     */
    static public function getTermFromId(int $term_id):mixed{

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_cat'
                    AND OvMdW_terms.`term_id` = '".$db->sanitizeData($term_id)."';
        ");

        $result = $db->plainQuery($query);
        if($result['cant_rows'] < 1){
            return new Error("Category", "There is no category with the provided term_id");
        }

        return new Category($result[0]);

    }
    
    /**
     * Returns a Category|Tag class that matches with the provided name and type.
     *
     * @param string $name
     * @param string $type
     * @return Category|Error
     */
    static public function getTermFromName(string $name):mixed{

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_cat'
                    AND OvMdW_terms.`name` = '".$db->sanitizeData($name)."';
        ");

        $result = $db->plainQuery($query);
        if($result['cant_rows'] < 1){
            return new Error("Category", "There is no category with the provided name");
        }

        return new Category($result[0]);

    }

    static public function getTermFromSlug(string $slug):Category|Error{

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_cat'
                    AND OvMdW_terms.`slug` = '".$db->sanitizeData($slug)."';
        ");

        $result = $db->plainQuery($query);
        if($result['cant_rows'] < 1){
            return new Error("Category", "There is no category with the provided slug");
        }

        return new Category($result[0]);

    }

}