<?php

namespace BulkUpload\Traits;

use Shuchkin\SimpleCSV;
use Shuchkin\SimpleXLS;
use Shuchkin\SimpleXLSX;
use Shuchkin\SimpleXLSXEx;
use Shuchkin\SimpleXLSXGen;

trait FileProcessor {

    /**
     * Converts the file in the address built by $fileFolder, $fileIdentifier and $extension to CSV. Returns the new CSV file address.
     *
     * @param string $fileFolder
     * @param string $fileIdentifier
     * @param string $extension
     * @return string
     */
    public function fileToCSV(array $rawData):string{
        
        return SimpleCSV::export($rawData);

    }

    /**
     * Converts the file in the address built by $fileFolder, $fileIdentifier and $extension to XLSX. Returns the new XLSX file address.
     *
     * @param string $fileFolder
     * @param string $fileIdentifier
     * @param string $extension
     * @return string
     */
    public function fileToXLSX(array $rawData, string $name):bool{
        
        $file = SimpleXLSXGen::fromArray($rawData);
        return $file->downloadAs($name);

    }

    /**
     * Returns the file extension of the uploaded file.
     *
     * @param string $fileFolder
     * @param string $fileIdentifier
     * @return string
     */
    public function getFileExtension(string $fileAddress):string{

        $i = strrpos($fileAddress, ".");
        if (!$i) {
            return "";
        }
        $l = strlen($fileAddress) - $i;
        $ext = strtolower(substr($fileAddress, $i + 1, $l));
        return $ext;

    }

    /**
     * Receives the $fileaddress and generates an array of the data to retrieve it.
     *
     * @param string $fileAddres
     * @return array
     */
    public function getFileData(string $fileAddress, string $separator = ";"):array{

        switch ($this->getFileExtension($fileAddress)) {

            case 'csv':

                if ($separator != ";") {
                    return SimpleCSV::import($fileAddress, false, $separator);
                } else {
                    return SimpleCSV::import($fileAddress);
                    break;
                }

            case 'xlsx':
                return SimpleXLSX::parse($fileAddress)->rows();
                break;

            case 'xls':
                return SimpleXLS::parse($fileAddress)->rows();
                break;
        }

    }

}