<?php

namespace  InventoryManagement;

trait InventoryCronVendor{

    /**
     * Sends an email in the case that after the 15 days, the vendor didn't confirm the stock of the published products. Returns true if the emails was sent succesfully.
     *
     * @param integer $daysPassed
     * @param string $vendorName
     * @param integer $productCount
     * @param integer $daysToValidate
     * @return boolean
     */
    public function productsToDraftNotification(int $daysPassed, string $vendorName, string $vendorEmail, int $productCount, int $daysToValidate = 15):bool{

        if($vendorName == ''){
            throw new \Exception("No fue proporcionado ningún nombre de vendedor.", 1);
        }

        if($vendorEmail == ''){
            throw new \Exception("No fue proporcionado un email de vendedor válido.", 1);
        }

        if($productCount < 1){
            throw new \Exception("No hay productos que revisar.", 1);
        }

        if($daysPassed < $daysToValidate){

            return false;

        }

        $content = "Estimado vendedor ".$vendorName.", no obtuvimos respuesta en la última revisión de inventario que le solicitamos. Por ello, de manera preventiva sus productos serán ocultados de nuestra plataforma. Por favor, póngase en contacto con nosotros en caso de tener alguna duda catalogo@bridgmarkets.com .";

        $mailer = new \email();

        $mailer->set_destination($vendorEmail);
        $mailer->set_template('Productos no actualizados', $content);

        if (!$mailer->send_mail()){
            return false;
        }

        return true;

    }

    /**
     * Sends an email that reminds how many days until the automatic change to draft occurs.  Returns true if the emails was sent succesfully.
     *
     * @param integer $daysPassed
     * @param string $vendorName
     * @param string $vendorEmail
     * @param integer $daysToValidate
     * @return boolean
     */
    public function inventoryManageReminder(int $daysPassed, string $vendorName, string $vendorEmail, string $identifierHash ,int $daysToValidate = 15):bool{

        if($vendorName == ''){
            throw new \Exception("No fue proporcionado ningún nombre de vendedor.", 1);
        }

        if($vendorEmail == ''){
            throw new \Exception("No fue proporcionado un email de vendedor válido.", 1);
        }
        if($daysPassed != 5 && $daysPassed != 10){
            return false;
        }

        $remainingDays = $daysToValidate - $daysPassed;

        $content = "Estimado vendedor ".$vendorName.", recuerde que le quedan ".$remainingDays." días para certificar el inventario disponible que posee en nuestra plataforma. Puede hacerlo <a href='".APP_ROOT."pages/inventory-management.php?identifier=".$identifierHash."'>aquí</a>. <br><br> Recuerde que si no realiza ninguna acción, los productos que tiene actualmente publicados serán pasados automáticamente a estado Borrador y dejarán de estar visibles para el usuario comprador. ";

        $mailer = new \email();

        $mailer->set_destination($vendorEmail);
        $mailer->set_template('Revisión de disponibilidad de productos', $content);

        if (!$mailer->send_mail()){
            return false;
        }

        return true;

    }

    public function inventoryManageLister(string $wp_api_root, string $vendorName, string $vendorEmail, string $identifierHash, $daysRemaining = 15):bool{

        if($vendorName == ''){
            throw new \Exception("No fue proporcionado ningún nombre de vendedor.", 1);
        }

        if($vendorEmail == ''){
            throw new \Exception("No fue proporcionado un email de vendedor válido.", 1);
        }

        if($wp_api_root == ''){
            throw new \Exception("No hay dirección válida al módulo de vendedor.", 1);
        }

        $content = "Estimado vendedor ".$vendorName.", por favor, marque las casillas de los productos que mantenga disponibles y deje vacía la casilla de aquellos de los que NO tiene inventario.<br><br>
        
        Los productos que NO tenga disponibles serán ocultados de nuestro sitio web para no dar lugar a órdenes de compra que no se puedan satisfacer.<br><br>

        En caso de no realizar el procedimiento de actualización de inventario que se facilita en <a href='".APP_ROOT."pages/inventory-management.php?identifier=".$identifierHash."'>este link</a> en el plazo de ".$daysRemaining." días (naturales), todos los productos que aparecen en el mismo serán ocultados de nuestra plataforma.<br><br> 
        
        Comprenda que para que el cliente obtenga una buena experiencia de compra es importante que todas sus solicitudes se puedan completar con éxito. <br><br>
       
        Si por el contrario quiere volver a hacer visible alguno de sus productos que alguna vez estuvo publicado recuerde que puede hacerlo a través de su <a href='".$wp_api_root.'dashboard'."'>módulo de vendedor</a>. <br><br>
       
        Para cualquier otra consulta, por favor, póngase en contacto con nuestro Departamento de catálogo: catalogo@bridgmarkets.com";

        $mailer = new \email();

        $mailer->set_destination($vendorEmail);
        $mailer->set_template('Manejo de inventario', $content);

        if (!$mailer->send_mail()){
            return false;
        }

        return true;

    }

}