<?php
class Orders
{
    private $user_id = 0;
    private $db;
    private $list = [];
    private $id_list = [];
    private $children = [];
    private $full_list = [];
    private $product_info = [];
    private $product_arr = [];
    private $queneau_list = [];
    private $meta_fields = [
        'first_last_name_account_owner' => 'first_last_name_account_owner',
        'optional_account_owner_company_name' => 'optional_account_owner_company_name',
        'account_owner_banks_name' => 'account_owner_banks_name',
        'account_owner_bank_number' => 'account_owner_bank_number',
        'bank_account_transfer_number' => 'bank_account_transfer_number',
        'transfer_date_payment' => 'transfer_date_payment',
        '_payment_method_title' => '_payment_method_title',
        '_payment_method' => '_payment_method',
        '_order_shipping' => '_order_shipping',
        '_order_tax' => '_order_tax',
        '_order_total' => '_order_total',
        '_user_information_payment' => '_user_information_payment',
        '_transaction_id' => '_transaction_id',
    ];


    public function __construct($user_id)
    {
        if (!isset($user_id) || empty($user_id)) {
            throw new Exception("Usuario no puede ser nulo", 501);
            return false;
        }
        $this->user_id = $user_id;
        $this->db = new collection();
    }

    public function getAll($page = 1, $count = 10)
    {
        $this->search_routine($page, $count);
        $this->compact();
        return array('orders' => $this->list, 'count' => count($this->id_list));
    }

    public function getDetails($order_id)
    {
        $this->get_order($order_id);
        $this->load_Products();
        $this->has_children($order_id);
        $this->load_alerts();
        $this->load_msg();
        $this->compact();
        return array('orders' => $this->list, 'count' => count($this->id_list));
    }



    private function load_alerts()
    {
        foreach ($this->full_list as $key => $value) {
            if ($value['shipping'] == 'DHL Express') {
                $this->full_list[$key]['alerts'][] = 'Los impuestos y pagos de aduanas en su destino deben correr por cuenta del comprador. Para conocer más visita nuestros <a  target="_blank" href="/informacion/terminos-y-condiciones">términos y condiciones.</a>';
            }
            if ($value['status'] == 'wc-capturing' && $value['details']['info']['_payment_method'] == 'bacs' && !$value['details']['info']['_user_information_payment']) {
                $this->full_list[$key]['alerts'][] = 'Por favor, incluya la información de pago de su transferencia bancaria para que podamos comenzar a tramitar su pedido.';
            }
        }
    }
    private function load_msg()
    {
        foreach ($this->full_list as $key => $value) {
            if ($value['status'] == 'wc-processing' && $value['details']['info']['_payment_method'] == 'bacs') {
                $this->full_list[$key]['msg'][] = 'Su pedido ha sido recibido, una vez el proveedor nos confirme inventario le haremos llegar vía mail la información bancaria para que complete la transferencia. Muchas gracias.';
            }
        }
    }

    private function load_products()
    {
        foreach ($this->full_list as $key => $value) {
            $this->full_list[$key]['details']['products'] = $this->loadProducts($key);
        }
    }

    private function search_routine($page = 1, $count = 10)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
                OvMdW_posts.ID,
                OvMdW_posts.post_date,
                OvMdW_posts.post_title,
                OvMdW_posts.post_status,
                OvMdW_posts.post_parent,
                GROUP_CONCAT(
                    OvMdW_woocommerce_order_items.order_item_name
                ) AS shipping,
                GROUP_CONCAT(OvMdW_postmeta.meta_value) AS total
            FROM
                OvMdW_posts
            INNER JOIN OvMdW_woocommerce_order_items ON OvMdW_woocommerce_order_items.order_id = OvMdW_posts.ID
            INNER JOIN OvMdW_postmeta ON OvMdW_postmeta.post_id = OvMdW_posts.ID
            WHERE
            OvMdW_posts.ID IN (SELECT
            GROUP_CONCAT(OvMdW_postmeta.post_id)
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_customer_user' AND
            OvMdW_postmeta.meta_value = '" . $this->user_id . "' GROUP BY
                OvMdW_postmeta.post_id) AND
            OvMdW_posts.post_type = 'shop_order' AND
            OvMdW_posts.post_parent = 0 AND
            OvMdW_woocommerce_order_items.order_item_type = 'shipping' AND
            OvMdW_postmeta.meta_key = '_order_total'
            GROUP BY
                OvMdW_posts.ID
            ORDER BY
                OvMdW_posts.ID DESC

            "
        );

        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function get_order($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_posts.ID,
            OvMdW_posts.post_date,
            OvMdW_posts.post_title,
            OvMdW_posts.post_status,
            OvMdW_posts.post_parent,
            GROUP_CONCAT(OvMdW_woocommerce_order_items.order_item_name) AS shipping,
            GROUP_CONCAT(OvMdW_postmeta.meta_value) AS total
            FROM
            OvMdW_posts
            INNER JOIN OvMdW_woocommerce_order_items ON OvMdW_woocommerce_order_items.order_id = OvMdW_posts.ID
            INNER JOIN OvMdW_postmeta ON OvMdW_postmeta.post_id = OvMdW_posts.ID
            WHERE
            OvMdW_posts.ID = '" . $order_id . "' AND
            OvMdW_woocommerce_order_items.order_item_type = 'shipping' AND
            OvMdW_postmeta.meta_key = '_order_total'
            GROUP BY
            OvMdW_posts.ID
            ORDER BY
            OvMdW_posts.ID DESC
            LIMIT 1
            "
        );
        $data = $this->db->plainQuery($query, '', true);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function get_info($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_postmeta.meta_key,
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key IN ('" . implode("','", $this->meta_fields) . "')  
            AND
            OvMdW_postmeta.post_id = " . $order_id . ""
        );
        $data = $this->db->plainQuery($query, false, true);
        $response = [];
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $response[$data[$i]['meta_key']] = $data[$i]['meta_value'];
        }
        return $response;
    }
    private function get_vendor_info($order_id)
    {
        $query1 = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_postmeta.meta_key,
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            (
            OvMdW_postmeta.meta_key = '_dokan_vendor_id'
            ) AND
            OvMdW_postmeta.post_id = " . $order_id . ""
        );
        $datan = $this->db->plainQuery($query1, '', true);
        if (!isset($datan[0]['meta_value']) || $datan[0]['meta_value'] == '') {
            return [];
        }
        $query2 = "SELECT
        OvMdW_users.ID,
        OvMdW_users.user_nicename,
        OvMdW_usermeta.meta_value
        FROM
        OvMdW_users
        INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
        WHERE
        OvMdW_usermeta.meta_key = 'dokan_store_name' AND
        OvMdW_usermeta.user_id = '" . $datan[0]['meta_value'] . "'           
        ";
        //die($query2);
        $data = $this->db->plainQuery($query2);
        $response = [];
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $response['vendor'] = $data[$i]['meta_value'];
            $response['vendor_slug'] = $data[$i]['user_nicename'];
            $response['vendor_id'] = $data[$i]['ID'];
        }
        return $response;
    }
    private function get_order_billing($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_postmeta.meta_key,
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key LIKE '_billing_%' AND
            OvMdW_postmeta.post_id = " . $order_id . "
            "
        );
        $data = $this->db->plainQuery($query, '', true);
        $response = [];
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $response[$data[$i]['meta_key']] = $data[$i]['meta_value'];
        }
        return $response;
    }
    private function get_order_shipping($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_postmeta.meta_key,
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key LIKE '_shipping_%' AND
            OvMdW_postmeta.post_id = " . $order_id . "
            "
        );
        $data = $this->db->plainQuery($query, '', true);
        $response = [];
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $response[$data[$i]['meta_key']] = $data[$i]['meta_value'];
        }
        return $response;
    }

    private function has_children($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_posts.ID,
            OvMdW_posts.post_date,
            OvMdW_posts.post_title,
            OvMdW_posts.post_status,
            OvMdW_posts.post_parent,
            GROUP_CONCAT(OvMdW_postmeta.meta_value) AS total
            FROM
            OvMdW_posts
            LEFT JOIN OvMdW_postmeta ON OvMdW_postmeta.post_id = OvMdW_posts.ID
            WHERE
            OvMdW_posts.post_parent = '" . $order_id . "' AND
            OvMdW_postmeta.meta_key = '_order_total'
            GROUP BY
            OvMdW_posts.ID
            ORDER BY
            OvMdW_posts.ID ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $key = $data[$i]['ID'];
            $this->children[$key]['order_id'] = $data[$i]['ID'];
            $this->children[$key]['name'] = $data[$i]['post_title'];
            $this->children[$key]['status'] = $data[$i]['post_status'];
            $this->children[$key]['created_at'] = $data[$i]['post_date'];
            $this->children[$key]['total'] = $data[$i]['total'];
            $this->children[$key]['post_parent'] = $data[$i]['post_parent'];
            $this->children[$key]['details']['products'] = $this->loadProducts($key);
            $this->children[$key]['details']['vendor'] = $this->get_vendor_info($key);
        }
        unset($data);
        unset($query);
        //print_r($this->children);
        return $this;
    }
    public function get_product_info($arr)
    {

        $products = new Products;
        $info = $products->searchById($arr, true);
        foreach ($info as $key => $value) {
            $this->product_info[$value['product_id']] = $value;
        }
    }

    private function loadProducts($order_id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_woocommerce_order_items.order_item_id,
            OvMdW_woocommerce_order_items.order_item_name,
            OvMdW_woocommerce_order_items.order_item_type,
            OvMdW_woocommerce_order_itemmeta.meta_key,
            OvMdW_woocommerce_order_itemmeta.meta_value
            FROM
            OvMdW_woocommerce_order_items
            INNER JOIN OvMdW_woocommerce_order_itemmeta ON OvMdW_woocommerce_order_itemmeta.order_item_id = OvMdW_woocommerce_order_items.order_item_id
            WHERE
            OvMdW_woocommerce_order_items.order_id = " . $order_id . " AND
            OvMdW_woocommerce_order_items.order_item_type = 'line_item'
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        $arr = [];
        $products = [];
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $key = $data[$i]['order_item_id'];
            $products[$key]['order_item_name'] = $data[$i]['order_item_name'];
            $products[$key][$data[$i]['meta_key']] = $data[$i]['meta_value'];
            if ($data[$i]['meta_key'] == '_product_id') {
                array_push($arr, $data[$i]['meta_value']);
            }
        }
        //die(print_r($products));
        if (empty($this->product_info)) {
            $this->get_product_info($arr);
        }
        $products_result = [];
        foreach ($products as $key => $value) {
            $value['info'] = $this->product_info[$value['_product_id']];
            $products_result[] = $value;
        }
        unset($data);
        unset($query);
        unset($products);
        unset($arr);
        return $products_result;
    }

    private function fillPrimaryList($data, $i)
    {
        $key = $data[$i]['ID'];
        $this->id_list[$key]['order_id'] = $data[$i]['ID'];
        $this->id_list[$key]['name'] = $data[$i]['post_title'];
        $this->id_list[$key]['status'] = $data[$i]['post_status'];
        $this->id_list[$key]['created_at'] = $data[$i]['post_date'];
        $this->id_list[$key]['shipping'] = $data[$i]['shipping'];
        $this->id_list[$key]['total'] = $data[$i]['total'];
        $this->id_list[$key]['post_parent'] = $data[$i]['post_parent'];
        $this->id_list[$key]['details'] = [];
        $this->id_list[$key]['details']['shipping'] = $this->get_order_shipping($key);
        $this->id_list[$key]['details']['billing'] = $this->get_order_billing($key);
        $this->id_list[$key]['details']['info'] = $this->get_info($key);
        $this->id_list[$key]['details']['vendor'] = $this->get_vendor_info($key);
        $this->id_list[$key]['details']['children'] = [];
        $this->id_list[$key]['msg'] = [];
        $this->id_list[$key]['alerts'] = [];
    }

    private function compact()
    {
        foreach ($this->children as $key => $value) {
            $this->full_list[$value['post_parent']]['details']['children'][] = $value;
        }

        $parent = [];
        foreach ($this->full_list as $key => $value) {
            $parent[$key][] = $value;
        }


        $this->full_list = [];
        foreach ($parent as $key => $value) {
            if ($value[0]['name'] != '') {
                array_push($this->list, $value[0]);
            }
        }
        $parent = [];
    }

    /**
     * @param number $order_id unique identifier of order
     * @param Array $metadata array of metadata for the order in format [key_name=>value_of_key]
     * @return boolean true if the order successfully updated, false otherwise
     */
    public function setOrderMeta($order_id, array $metadata): bool
    {
        if (!isset($order_id) || $order_id == '') {
            throw new Exception("Error Processing Request", 1);
            return false;
        }
        $existing = [];
        if (count($metadata) > 0) {
            $query = str_replace(
                'ovmdw_',
                TABLE_PREFIX,
                "SELECT
                OvMdW_postmeta.meta_id,
                OvMdW_postmeta.meta_key,
                OvMdW_postmeta.meta_value
                FROM
                OvMdW_postmeta
                WHERE
                OvMdW_postmeta.meta_key IN ('" . implode("','", array_keys($metadata)) . "')  
                AND
                OvMdW_postmeta.post_id = " . $order_id . ""
            );
            $data = $this->db->plainQuery($query, false, true);
            for ($i = 0; $i < $data['cant_rows']; $i++) {
                $existing[$data[$i]['meta_key']] = $data[$i]['meta_id'];
            }
            foreach ($metadata as $key => $value) {
                if (isset($existing[$key])) {
                    $this->db->update('OvMdW_postmeta', $existing[$key], array("meta_value" => $value));
                } else {
                    $this->db->insert('OvMdW_postmeta', array("meta_key" => $key, "meta_value" => $value, "post_id" => $order_id));
                }
            }
            return true;
        }



        return false;
    }

    public function get_order_meta(Array $order,string $key)
    {
        
        return false;
    }

    public function sendEmailNotificationToAdministration($order_id,$fullName='')
    {
        $order = $this->getDetails($order_id)['orders'][0];
        $row['id'] = $order_id;
        $row['client'] = ($order['details']['info']['first_last_name_account_owner']!='')?$order['details']['info']['first_last_name_account_owner']:$order['details']['billing']['_billing_first_name'] . ' ' . $order['details']['billing']['_billing_last_name'];
        $row['banco'] =$order['details']['info']['account_owner_banks_name'];
        $row['cuenta'] = $order['details']['info']['account_owner_bank_number'];
        $row['transfer'] = $order['details']['info']['bank_account_transfer_number'];
        $row['date'] = $this->dateTransferToEmail($order['details']['info']['transfer_date_payment']);
        $this->bi_waiting_for_capture_notification(array($row),$order_id,$fullName,$order['total']);
    }
    
    private function dateTransferToEmail($date)
    {
        $d = explode('-', $date);
        return  $d[2] . '-' . $d[1] . '-' . $d[0];
    }

    private function GenerateTableBodyFromArray(array $pending_order): String
    {
        $tbody = '<tbody>';
        foreach ($pending_order as $key => $row) {
            $tbody .= '<tr>';
            foreach ($row as $key => $val) {
                $tbody .= '<td>' . $val . '</td>';
            }
            $tbody .= '</tr>';
        }
        $tbody .= '</tbody>';
        return $tbody;
    }

    function bi_waiting_for_capture_notification($orders_mail_list,$id,$name,$amount)
    {
        $replace_vars = array(
            '{{title}}' => utf8_decode('Pedidos pendientes por gestión de pagos'),
            '{{mail_body}}' => utf8_decode('Srs. Bridgemarkets el usuario '.$name.' ha diligenciado el formulario para el pedido '.$id.' por valor de USD$ '.$amount.'. Por favor, revise la siguiente información y proceda con la respectiva gestión.'),
            '{{mail_table}}' => $this->GenerateTableBodyFromArray($orders_mail_list),
        );
        //setting BM mail notification
        $email= new email();
        $email->set_multidestination(ADMIN_EMAIL_NOTIFICATIONS);
        $email->load_template('Pedidos pendientes de gestión de pagos','email/complete_transfer_data_form');
        $email->setReplace($replace_vars);
        $email->send_mail();
    }
}
