<?php
class Payment
{

    public $db;
    public $user_id;
    private $providers;
    private Destination $destination;
    private string $payment_method;
    public PaymentMethod $class;
    private array $response = [];
    public array $errors = [];

    public function __construct($user)
    {
        $this->db = new collection();
        $this->user_id = $user;
    }

    /**
     * load registered shipping providers from database
     * 
     * @return void
     */
    private function registeredProviders()
    {
        $query = "
        SELECT
            bm_payment_method.payment_method,
            bm_payment_method.payment_method_title,
            bm_payment_method.set_pay,
            bm_payment_method.class_name,
            bm_payment_method.id
        FROM
            bm_payment_method
        INNER JOIN rel_country_payment ON rel_country_payment.id_provider = bm_payment_method.id
        LEFT JOIN bi_country_table ON bi_country_table.id_country = rel_country_payment.id_country
        WHERE
            (bi_country_table.country_name = '" . $this->db->sanitizeData($this->destination->country) . "'
        OR rel_country_payment.id_country = 0) AND 
        bm_payment_method.id NOT IN(
        SELECT
        a.id_provider
        FROM
        rel_country_payment AS a
        INNER JOIN bi_country_table AS b ON b.id_country = a.exclude
        WHERE
        b.country_name = '" . $this->db->sanitizeData($this->destination->country) . "'
        GROUP BY
        a.id
        )
        GROUP BY
            bm_payment_method.id
        ";
        //die($query);
        $data = $this->db->plainQuery($query);

        foreach ($data as $provider) {
            if ($provider['payment_method']) {
                if (class_exists($provider['class_name'])) {
                    $this->providers[$provider['payment_method']] = $provider;
                }
            }
        }
        return $this;
    }
    /**
     * upload data from json file
     * @param Array $data
     * 
     * @return void
     */
    public function loadData(array $data)
    {
        if (!isset($data['address'])) {
            throw new Exception("Formato de dirección inválido", 2);
        }
        $this->setDestination($data['address']);
        $this->registeredProviders();
        $this->response['providers'] = [];
        foreach ($this->providers as $key => $provider) {
            if ($provider['class_name'] != '' && class_exists($provider['class_name']) && $provider['class_name'] != 'WompiLink') {
                $method = new $provider['class_name']($this->user_id);
                if ($method instanceof PaymentMethod) {
                    $res = $method->loadProvider($this->destination);
                    if (!empty($res)) {
                        $res['payment_method'] = $key;
                        $res['payment_method_title'] = $provider['payment_method_title'];
                        $res['order_payment_sale_condition'] = $provider['set_pay'];
                        $this->response['providers'][] = $res;
                    }
                    if (!$method->getStatus()) {
                        $this->errors[$key] = $method->error;
                    }
                }
            }
        }
        return $this;
    }
    /**
     * upload data from json file
     * @param Array $data
     * 
     * @return void
     */
    public function loadMethod()
    {
        if (!isset($this->payment_method)) {
            throw new Exception("acceso Restringido", 2);
        }
        $this->response['providers'] = [];
        $data = $this->db->search('bm_payment_method', array('payment_method' => $this->payment_method));

        if ($data[0]['payment_method']) {
            $res['payment_method'] = $data[0]['payment_method'];
            $res['payment_method_title'] = $data[0]['payment_method'];
            $res['order_payment_sale_condition'] = $data[0]['set_pay'];
            $res['options'] = $this->class->loadOptions();
            $this->response['providers'][] = $res;
        }
        return $this;
    }

    public function setProvider(string $payment_method)
    {
        $this->payment_method = $payment_method;
        $data = $this->db->search('bm_payment_method', array('payment_method' => $this->payment_method));

        if ($data[0]['payment_method']) {
            if (class_exists($data[0]['class_name'])) {
                $this->class = new $data[0]['class_name']($this->user_id);
            }
        }
    }

    /**
     * get response data from object as array
     * 
     * @return array
     */

    public function getResponse()
    {
        return $this->response;
    }

    /**
     * set current destination
     * 
     * @return void
     */
    private function setDestination($destination)
    {
        if (!is_a($destination, 'Destination')) {
            try {
                $this->destination = new Destination($destination);
            } catch (\Throwable $th) {
                throw new Exception("Formato de dirección inválido", 2);
            }
        }
        return $this;
    }
}
