<?php

class GlobalCargo_class implements Currier
{

    private $db;
    private Destination $destination;
    private Cart $cart;
    private Package $package;
    private $total = 0;
    public $totalTransitDays = 25;
    public $RangeTransitDays = '11 - 14 días hábiles';
    public $bank_fee = 0.03;


    public $cbm_cost = [
        'Costa Rica' => [
            'min' => 1,
            'cost' => 65,
            'bank_fee' => 0.03,
            'transitDays' => '11 - 14 días hábiles',
            'message' => 'Notar que el tiempo presentado no incluye desconsolidación en destino, ya que el INCOTERM utilizado para este envío es FAS (Free alonside ship): su mercancía será entregada en un almacén fiscal en destino.',
            'productName' => 'Envió terrestre de 11 - 14 días hábiles. El tiempo indicado es aproximado y está sujeto a verificación de inventario por parte del proveedor y el plazo de entrega del operador logístico.',
            'MaxTransitDays' => 14,
            'taxes' => [
                [
                    'id' => 'EFR',
                    'name' => 'EFR',
                    'cost' => 40
                ],
                [
                    'id' => 'CP',
                    'name' => 'Carta Porte',
                    'cost' => 45
                ],
                [
                    'id' => 'DUCA',
                    'name' => 'DUCA',
                    'cost' => 40
                ],
            ]
        ],
        'Guatemala' => [
            'min' => 1,
            'cost' => 95,
            'bank_fee' => 0.03,
            'transitDays' => '14 - 17 días hábiles',
            'message' => 'Su mercancía estará disponible para retiro en almacén Fiscal bajo el Incoterm DPU, los costos en destino son asumidos por el cliente final. (Impuestos y manejos).',
            'productName' => 'Tenga en cuenta que el periodo de despacho es de aproximadamente 25 días.',
            'MaxTransitDays' => 17,
            'taxes' => [
                [
                    'id' => 'FAS',
                    'name' => 'FAS',
                    'cost' => 40
                ],
                [
                    'id' => 'CP',
                    'name' => 'Carta Porte',
                    'cost' => 45
                ],
            ],
            'cft_price' => [
                [
                    'min' => 0,
                    'max' => 5,
                    'taxable' => false,
                    'pricing_type' => 'fixed',
                    'price' => 30
                ],
                [
                    'min' => 5,
                    'max' => 15.99999999999999,
                    'taxable' => true,
                    'price' => 6,
                ],
                [
                    'min' => 16,
                    'max' => 35.315,
                    'taxable' => true,
                    'pricing_type' => 'fixed',
                    'price' => 95
                ]
            ]
        ]
    ];
    public $logistic_cost = 0;
    public $rates = null;
    public $shipping = 0;
    public $error = '';
    public $response = [];
    public $status = true;
    private $ranges = [];
    private $price_based_on = 'cbm';

    function __construct()
    {
        $this->db = new collection;
    }

    function generateResponseSchema(array $data, bool $include_taxes = false)
    {
        $res = [];
        $res['method_title'] = 'Global Cargo';
        $res['description'] = 'Global Cargo';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['message'] = $this->rates['message'];
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/globalcargo.png';
        $prod['type'] = 'ground';
        $prod['productName'] = $this->rates['productName'];
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = round($this->shipping + $this->getBankFeeTotal(), 2);
        $prod['volumetric'] = $this->package->getTotalCBM();
        $prod['deliveryCapabilities']['totalTransitDays'] = $this->totalTransitDays;
        $prod['details'][] = $this->insertDetail('Envío', '', $this->shipping + $this->getBankFeeTotal());
        //$prod['details'][] = $this->insertDetail('CBM', '', $this->package->getTotalCBM()['ground']);
        //$prod['details'][] = $this->insertDetail('Peso', '', $this->package->getTotalWeight());
        //$prod['details'][] = $this->insertDetail('Trámite logístico', 0, $this->logistic_cost);
        //$prod['details'][] = $this->insertDetail('Impuesto de Nacionalización', 'Cargos por nacionalización', 0);
        $res['options'][] = $prod;
        return $res;
    }

    private function getBankFeeTotal(): float
    {
        if(!isset($this->rates['cft_price'])){
            return 0;
        }
        //die(var_dump($this->shipping));
        return ($this->shipping + $this->package->getTotalOrder()) * $this->bank_fee;
    }

    private function calculateLogisticAmount(): void
    {
        if (isset($this->cbm_cost[$this->destination->country])) {
            $this->rates = $this->cbm_cost[$this->destination->country];
            $this->RangeTransitDays = $this->rates['transitDays'];
            $this->totalTransitDays = $this->rates['MaxTransitDays'];
            $this->shipping = $this->getFinalShippingPrice();
            //die(var_dump($this->shipping));
        }
    }

    private function getFinalShippingPrice(): float
    {
      //die(var_dump( $this->rates));
        $total = 0;
        $cbm = $this->package->getTotalCBM()['ground'];
        //die(var_dump($this->package->getTotalCBM()));
        if ($cbm > $this->rates['min']) {
            foreach ($this->rates['taxes'] as $key => $value) {
                $total += $value['cost'];
            }
            $this->logistic_cost = $total;

            return $cbm * $this->rates['cost'] +$this->logistic_cost ;
        } else if (!isset($this->rates['cft_price'])) {
            
            foreach ($this->rates['taxes'] as $key => $value) {
                $total += $value['cost'];
            }
            $this->logistic_cost = $total;
           // die(var_dump( $this->rates));
            return $this->rates['cost'] + $this->logistic_cost;
        } else {
            $cft = $cbm * 35.315;
                        
            if ($cft <= 5) {
                $this->logistic_cost = 30;
                
                return 30;
            } else if ($cft > 5 && $cft < 16) {
                foreach ($this->rates['taxes'] as $key => $value) {
                    $total += $value['cost'];
                }
                //die(var_dump($cbm));
                $this->logistic_cost = $total;
                return $cft * 6 + $this->logistic_cost;
            } else if ($cft >= 16 && $cft <= 35.315) {
                foreach ($this->rates['taxes'] as $key => $value) {
                    $total += $value['cost'];
                }
                $this->logistic_cost = $total;
                return 95 +$this->logistic_cost ;
            }
        }
    }

    private function getCftPricingInfo(): array
    {
        $cft = $this->package->getTotalCBM()['ground'] * 35.315;
        $result = [];
        foreach ($this->rates['cft_price'] as $key => $value) {
            if ($cft <= $value['max']) {
                return  $value;
            }
        }
        return $result;
    }

    private function getRealShippingCost(): float
    {
        $cbm = $this->package->getTotalCBM()['ground'];
        $cft = round($cbm * 35.315, 4);

        if (count($this->ranges) < 1) {
            return ($cbm > $this->rates['min']) ? $cbm * $this->rates['cost'] : $this->rates['cost'];
        }
        foreach ($this->ranges as $range) {
            if ($cft >= $range['min'] && ($cft <= $range['max'] || $range['max'] == 0)) {
                return $this->typeBasedCalculator($range, $cft, $cbm);
            }
        }
        return ($cbm > $this->rates['min']) ? $cbm * $this->rates['cost'] : $this->rates['cost'];
    }

    private function typeBasedCalculator(array $range, float $cft = 0, float $cbm = 0):float
    {
        //die(var_dump([$range, $cft, $cbm]));
        switch ($range['measure']) {
            case 'fixed':
                $this->price_based_on = 'fixed';
                return $range['cost'];
                break;
            case 'cft':
                $this->price_based_on = 'range';
                //die(var_dump($$range['measure']));
                return $cft * $range['cost'];
                break;
            case 'cbm':
                $this->price_based_on = 'cbm';
                return $cbm * $range['cost'];
                break;
        }
        return 0;
    }

    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }

    public function errorResponse()
    {
        $res['error'] = true;
        $res['method_title'] = 'Global Cargo';
        $res['description'] = 'Global Cargo';
        $res['include_taxes'] = 'No incluye impuestos en destino';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/globalcargo.png';
        $res['message'] = $this->error;
        $prod['type'] = 'ground';
        $prod['productName'] = $this->error;
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = 0;
        $prod['volumetric'] = $this->package->getTotalCBM();
        $prod['deliveryCapabilities']['totalTransitDays'] = 0;
        $res['options'][] = $prod;
    }



    private function recalculate()
    {
        $this->total = 0;
        $this->calculateLogisticAmount();
        $this->total += $this->logistic_cost;
        $this->total += $this->shipping;
    }

    function checkout_rate(Destination $_item, Package $package, bool $include_taxes): array
    {
        $this->destination = $_item;
        $this->package = $package;
        $this->recalculate();
        if ($this->total != 0) {
            return $this->generateResponseSchema([], $include_taxes);
        }
        return [];
    }

    private function setRangesIfExist(): void
    {
        $provider = $this->db->search('bm_providers', ['methodNo' => 'globalcargo']);
        if ($provider['cant_rows'] < 1) {
            $this->ranges = [];
            $ranges_response['isTaxable'] = true;
        };
        $ranges_response = $this->db->search('bm_logistic_provider_cost_bracket', ['provider_id' => $provider[0]['id'], $this->destination->get_countryId()]);
        if ($ranges_response['cant_rows'] < 1) {
            $this->ranges = [];
            $ranges_response['isTaxable'] = true;
        };
        unset($ranges_response['cant_rows']);
        $this->ranges = $ranges_response;
    }
}
